var prefs = {};

let default_prefs = {
  //Identity
  showIdentity: false,
  //Attachments
  attachExtCheckEnable: false,
  attachExts: [],
  attachSizeLimit: 0,
  attachBlacklistCheck: true,
  //Words
  wordNGCheckEnable: false,
  ngWords: [],
  wordMustCheckEnable: false,
  mustWords: [],
  wordIsRegExp: false,
  wordCaseSense: false,
  wordIgnoreQuote: true,
  wordCheckBody: true,
  wordCheckSubject: false,
  //Recipient names
  recNameMode: 2, //0: correct, 1: remove, 2: none
  recNameRemoveNotInAb: false,
  recNameSuppressWarn: false,
  //Recipient types
  needTo: true,
  needCc: false,
  needBcc: false,
  needReplyTo: false,
  limitToCc: 0,
  //Address check
  checkDup: true,
  dupSuppressWarn: false,
  addrCheckMode: 0, //0: popup all, 1: addrbook/list, 3: none
  addrCheckAbEnable: false,
  addrCheckAb: "all",
  abWhite: false,
  addrCheckListEnable: false,
  addrList: [],
  listWhite: false,
  addrCheckExcludeBcc: false,
  addrCheckBoth: false
};

var currentIdentity = "common";

browser.runtime.onMessage.addListener(async message => {
  switch (message.message) {
    case "SEND_IDS_AND_ABS":
      buildAddressBookList(message.addressbooks);
      await buildIdentityList(message.identities);
      break;
    default:
      break;
  }
});

function buildAddressBookList(addressbooks) {
  let select = document.getElementById("addrCheckAb");
  for (let id in addressbooks) {
    let name = addressbooks[id];
    let option = document.createElement("option");
    option.value = id;
    option.textContent = name;
    select.add(option);
  }
}

async function buildIdentityList(identities) {
  let select = document.getElementById("identity");
  for (let id in identities) {
    let option = document.createElement("option");
    option.textContent = identities[id];
    option.setAttribute("value", id);
    select.add(option);
  }
  select.selectedIndex = 0;

  await switchPrefsForId();
  select.addEventListener("change", switchPrefsForId);
  document.cas_form.casEnable[0].addEventListener("change", saveCASModePref);
  document.cas_form.casEnable[1].addEventListener("change", saveCASModePref);
  document.cas_form.casEnable[2].addEventListener("change", saveCASModePref);
}

async function switchPrefsForId() {
  let select = document.cas_form.identity;
  let index = select.selectedIndex;
  let val = select.options[index].value;

  currentIdentity = val;

  if (val == "common") {
    document.getElementById("casEnableCommon").setAttribute("class", "box-collapse");
  } else {
    document.getElementById("casEnableCommon").setAttribute("class", "box-visible");
  }

  await initCASMode();
  await initPrefs();
}

async function initCASMode() {
  let select = document.cas_form.identity;
  let index = select.selectedIndex;
  let id = select.options[index].value;

  let mode = "cas_" + id + "_mode";
  let result = await browser.storage.local.get(mode);

  if (result && result[mode] < 3) {
    document.cas_form.casEnable[result[mode]].checked = true;
  } else if (id == "common") { //mode for common is not saved
    document.cas_form.casEnable[1].checked = true;
  } else { //mode for identity is not saved
    document.cas_form.casEnable[0].checked = true;
  }

  toggleConfigArea();
}

async function saveCASModePref() {
  toggleConfigArea();
  let str = "cas_" + currentIdentity + "_mode";
  let val = document.cas_form.casEnable.value;

  let obj = {};
  obj[str] = val;
  await browser.storage.local.set(obj);
}

function toggleConfigArea() {
  if (document.cas_form.casEnable[0].checked || document.cas_form.casEnable[2].checked) {
    document.getElementById("configArea").setAttribute("class", "box-collapse");
  } else {
    document.getElementById("configArea").setAttribute("class", "box-visible");
  }
}

async function initPrefs() {
  let prop = "cas_" + currentIdentity;
  let result = await browser.storage.local.get(prop);

  if (!result || !result[prop]) { //prefs is not recorded
    //copy default prefs
    for (let key in default_prefs) {
      prefs[key] = default_prefs[key];
    }
  } else {
    prefs = result[prop];
  }

  //for (let key in prefs) {
  for (let key in default_prefs) { //use default_prefs in case pref is added in the future version
    let elem = document.getElementById(key);

    if (!elem) {
      continue;
    }

    let type = elem.getAttribute("type");
    let val = prefs[key];

    switch (elem.localName) {
      case "input":
        if (type == "checkbox") {
          elem.checked = val ? true : false;
        } else if (type == "radio") {
          break;
        } else if (type == "number") {
          elem.value = val;
        } else {
          elem.value = val ? val : "";
        }
        break;
      case "select":
        if (key == "addrList" || key == "attachExts" || key == "ngWords" || key == "mustWords") {
          fillSelectOptions(elem, val)
          continue; //no need to add event listener. event is triggered when the button is clicked.
        } else if (key == "abWhite" || key == "listWhite" || key == "attachBlacklistCheck") {
          prefs[key] ? elem.value = "true" : elem.value = "false";
        } else {
          elem.value = val;
        }
        break;
      default:
        break;
    }

    elem.addEventListener("change", savePrefs);
  }

  //address check mode
  let index = prefs["addrCheckMode"];
  document.cas_form.addrCheckMode[index].checked = true;
  document.cas_form.addrCheckMode[0].addEventListener("change", savePrefs);
  document.cas_form.addrCheckMode[1].addEventListener("change", savePrefs);
  document.cas_form.addrCheckMode[2].addEventListener("change", savePrefs);

  //recipient name check mode
  index = prefs["recNameMode"];
  document.cas_form.recNameMode[index].checked = true;
  document.cas_form.recNameMode[0].addEventListener("change", savePrefs);
  document.cas_form.recNameMode[1].addEventListener("change", savePrefs);
  document.cas_form.recNameMode[2].addEventListener("change", savePrefs);
}

function fillSelectOptions(elem, val) {
  //clear existing optons
  let length = elem.length;
  for (let i = length - 1; i >= 0; i--) {
    elem.remove(i);
  }

  //add new ones
  for (let i = 0; i < val.length; i++) {
    let opt = document.createElement("option");
    opt.textContent = val[i];
    elem.add(opt);
  }
}

async function savePrefs() {
  let prop = "cas_" + currentIdentity;
  //for (let key in prefs) {
  for (let key in default_prefs) { //use default_prefs in case pref is added in the future version
    let elem = document.getElementById(key);

    if (!elem) continue;

    let type = elem.getAttribute("type");
    switch (elem.localName) {
      case "input":
        if (type == "checkbox") {
          prefs[key] = elem.checked ? true : false;
        } else if (type == "radio") {
          break;
        } else if (type == "number") {
          prefs[key] = Number.isNaN(elem.value) ? 0 : elem.value;
        } else {
          prefs[key] = elem.value ? elem.value : "";
        }
        break;
      case "select":
        if (key == "addrList" || key == "attachExts" || key == "ngWords" || key == "mustWords") {
          prefs[key] = getValueFromSelect(elem);
        } else if (key == "abWhite" || key == "listWhite" || key == "attachBlacklistCheck") {
          prefs[key] = (elem.value == "true");
        } else {
          prefs[key] = elem.value;
        }
        break;
      default:
        break;
    }
  }

  //address check mode
  prefs["addrCheckMode"] = Number.parseInt(document.cas_form.addrCheckMode.value);

  //recipient name check mode
  prefs["recNameMode"] = Number.parseInt(document.cas_form.recNameMode.value);

  let obj = {};
  obj[prop] = prefs;
  await browser.storage.local.set(obj);
}

function getValueFromSelect(elem) {
  let ret = [];
  let opts = elem.options;
  for (let i = 0; i < opts.length; i++) {
    let val = opts[i].textContent;
    if (val) ret.push(val);
  }

  return ret;
}

function addSelectionOption(selectId) {
  let select = document.getElementById(selectId);
  let opt = document.createElement("option");
  let textField = document.getElementById(selectId + "Input");
  if (textField.value) {
    opt.textContent = textField.value;
    select.add(opt);
    textField.value = "";
    savePrefs();
  }
}

function deleteSelectionOption(selectId) {
  let select = document.getElementById(selectId);
  let index = select.selectedIndex;
  let opts = select.childNodes;
  select.remove(index);
  savePrefs();
}

function registerEventListeners() {
  document.getElementById("addrListAdd").addEventListener("click", (event) => {
    addSelectionOption("addrList")
  });
  document.getElementById("addrListDel").addEventListener("click", (event) => {
    deleteSelectionOption("addrList")
  });

  document.getElementById("attachExtsAdd").addEventListener("click", (event) => {
    addSelectionOption("attachExts")
  });
  document.getElementById("attachExtsDel").addEventListener("click", (event) => {
    deleteSelectionOption("attachExts")
  });

  document.getElementById("ngWordsAdd").addEventListener("click", (event) => {
    addSelectionOption("ngWords")
  });
  document.getElementById("ngWordsDel").addEventListener("click", (event) => {
    deleteSelectionOption("ngWords")
  });

  document.getElementById("mustWordsAdd").addEventListener("click", (event) => {
    addSelectionOption("mustWords")
  });
  document.getElementById("mustWordsDel").addEventListener("click", (event) => {
    deleteSelectionOption("mustWords")
  });
}

async function init() {
  browser.runtime.sendMessage({
    message: "GET_IDS_AND_ABS"
  });

  registerEventListeners();
}

init();
